;				ACPA.ASM
;
; These routines allow initializing the ACPA DSP code and data RAM.
; All routines are IBM C far pascal for use in an OS/2 IOPL segment.
;
;   ACPAINIT	will verify that the ACPA card resides at the address
;		specified by _ACPABASE, and that all 8k memory functions
;		correctly.  The DSP code defined between labels DSPCODE
;		and DSPCODE_END will be loaded starting at DSP address 0.
;
;   ACPACLOSE	will reset the DSP
;
;   ACPAREAD	reads a word from anywhere in the ACPA shared memory
;
;   ACPAWRITE	writes a word to anywhere in the ACPA shared memory
;
;

DATAREGLOW	EQU	0
DATAREGHI	EQU	1
ADDREGLOW	EQU	4
ADDREGHI	EQU	5
HOSTCOMREG	EQU	6
HOSTSTATUSREG	EQU	6
CLEARACCESSERR	EQU	7

; Command Reg equates
CTRRES		EQU	080H
INTHOST		EQU	040H
DSPINTACK	EQU	020H
SPBINTACK	EQU	010H
INPSEL		EQU	008H
ADCEN		EQU	004H
DAC2EN		EQU	002H
DAC1EN		EQU	001H

; Status Reg equates
HOSTINT		EQU	8
ADCINT		EQU	4
DAC2INT		EQU	2
DAC1INT		EQU	1

; DSP Sample Memory equates
DAC1BLOCK0	EQU	4000H
DAC1BLOCK1	EQU	4100H
DAC2BLOCK0	EQU	4200H
DAC2BLOCK1	EQU	4300H
SCRBLOCK0	EQU	4400H
SCRBLOCK1	EQU	4500H
ADCBLOCK0	EQU	4600H
ADCBLOCK1	EQU	4700H

; Define IBM C/2 segments

_TEXT SEGMENT BYTE PUBLIC 'CODE'
_TEXT ENDS

NULL	SEGMENT PARA PUBLIC 'BEGDATA'
NULL	ENDS

_DATA	SEGMENT WORD PUBLIC 'DATA'
_DATA	ENDS

CONST SEGMENT WORD PUBLIC 'CONST'
CONST ENDS

_BSS SEGMENT WORD PUBLIC 'BSS'
_BSS ENDS

STACK	SEGMENT PARA STACK 'STACK'
STACK	ENDS

DGROUP GROUP NULL, _DATA,CONST,_BSS,STACK

	ASSUME	CS:_IOSEG,DS:DGROUP,ES:DGROUP,SS:DGROUP

_DATA	SEGMENT	PUBLIC

_ACPABASE DW	0		; This address set in SAMPLE2.C
	PUBLIC	_ACPABASE

; DSP OPCODE EQUATES FOR TMS320C25
AR	EQU	80H		; Data pointed to by current aux register
ARPLUS	EQU	0A0H		; " ", then inc AR
ARMINUS EQU	90H		; " ", then dec AR
ARSET0	EQU	88H		; " ", then set ARP = 0
ARSET1	EQU	89H		; " ", then set ARP = 1
ARSET2	EQU	8AH
ARSET3	EQU	8BH
ARSET4	EQU	8CH
ARSET5	EQU	8DH
ARSET6	EQU	8EH
ARSET7	EQU	8FH
ARSET	EQU	88H		; " ", then set ARP = "+n"
ARPLUS0 EQU	0A8H
ARPLUS1	EQU	0A9H
ARPLUS2	EQU	0AAH
ARPLUS3	EQU	0ABH
ARPLUS4	EQU	0ACH
ARPLUS5	EQU	0ADH
ARPLUS6	EQU	0AEH
ARPLUS7	EQU	0AFH
ARPLUSSET EQU	0A8H
ARMINUS0 EQU	98H
ARMINUS1 EQU	99H
ARMINUS2 EQU	9AH
ARMINUS3 EQU	9BH
ARMINUS4 EQU	9CH
ARMINUS5 EQU	9DH
ARMINUS6 EQU	9EH
ARMINUS7 EQU	9FH
ARMINUSSET EQU	98H
ARMINUSAR0 EQU	0D0H
ARMINUSAR0SET EQU 0D8H
ARPLUSAR0 EQU	0E0H
ARPLUSAR0SET EQU 0E8H

; * indicates 320C25 extensions, 2 = 2 word instructions
DSPABS	EQU	0CE1BH
ADD0	EQU	0000H
ADD1	EQU	0100H
ADD2	EQU	0200H
ADD3	EQU	0300H
ADD4	EQU	0400H
ADD5	EQU	0500H
ADD6	EQU	0600H
ADD7	EQU	0700H
ADD8	EQU	0800H
ADD9	EQU	0900H
ADD10	EQU	0A00H
ADD11	EQU	0B00H
ADD12	EQU	0C00H
ADD13	EQU	0D00H
ADD14	EQU	0E00H
ADD15	EQU	0F00H
ADDC	EQU	4300H	;*
ADDH	EQU	4800H
ADDK	EQU	0CC00H	;*
ADDS	EQU	4900H
ADDT	EQU	4A00H	;*
ADLK0	EQU	0D002H	;*2
ADLK1	EQU	0D102H	;*2
ADLK2	EQU	0D202H	;*2
ADLK3	EQU	0D302H	;*2
ADLK4	EQU	0D402H	;*2
ADLK5	EQU	0D502H	;*2
ADLK6	EQU	0D602H	;*2
ADLK7	EQU	0D702H	;*2
ADLK8	EQU	0D802H	;*2
ADLK9	EQU	0D902H	;*2
ADLK10	EQU	0DA02H	;*2
ADLK11	EQU	0DB02H	;*2
ADLK12	EQU	0DC02H	;*2
ADLK13	EQU	0DD02H	;*2
ADLK14	EQU	0DE02H	;*2
ADLK15	EQU	0DF02H	;*2
ADRK	EQU	7E00H	;*
DSPAND	EQU	4E00H
ANDK0	EQU	0D004H	;*2
ANDK1	EQU	0D104H	;*2
ANDK2	EQU	0D204H	;*2
ANDK3	EQU	0D304H	;*2
ANDK4	EQU	0D404H	;*2
ANDK5	EQU	0D504H	;*2
ANDK6	EQU	0D604H	;*2
ANDK7	EQU	0D704H	;*2
ANDK8	EQU	0D804H	;*2
ANDK9	EQU	0D904H	;*2
ANDK10	EQU	0DA04H	;*2
ANDK11	EQU	0DB04H	;*2
ANDK12	EQU	0DC04H	;*2
ANDK13	EQU	0DD04H	;*2
ANDK14	EQU	0DE04H	;*2
ANDK15	EQU	0DF04H	;*2
APAC	EQU	0CE15H
B	EQU	0FF80H
BACC	EQU	0CE25H	;*
BANZ	EQU	0FB80H
BBNZ	EQU	0F980H	;*2
BBZ	EQU	0F880H	;*2
BC	EQU	5E80H	;*2
BGEZ	EQU	0F480H
BGZ	EQU	0F180H
BIOZ	EQU	0FA80H
BIT0	EQU	9000H	;*
BIT1	EQU	9100H	;*
BIT2	EQU	9200H	;*
BIT3	EQU	9300H	;*
BIT4	EQU	9400H	;*
BIT5	EQU	9500H	;*
BIT6	EQU	9600H	;*
BIT7	EQU	9700H	;*
BIT8	EQU	9800H	;*
BIT9	EQU	9900H	;*
BIT10	EQU	9A00H	;*
BIT11	EQU	9B00H	;*
BIT12	EQU	9C00H	;*
BIT13	EQU	9D00H	;*
BIT14	EQU	9E00H	;*
BIT15	EQU	9F00H	;*
BITT	EQU	5700H	;*
BLEZ	EQU	0F280H
BLKD	EQU	0FD00H	;*2
BLKP	EQU	0FC00H	;*2
BLZ	EQU	0F380H
BNC	EQU	5F80H	;*2
BNV	EQU	0F780H	;*2
BNZ	EQU	0F580H
BV	EQU	0F080H
BZ	EQU	0F680H
CALA	EQU	0CE24H
DSPCALL	EQU	0FE80H
CMPL	EQU	0CE27H	;*
CMPREQ	EQU	0CE50H	;*
CMPRLT	EQU	0CE51H	;*
CMPRGT	EQU	0CE52H	;*
CMPRNE	EQU	0CE53H	;*
CNFD	EQU	0CE04H	;*
CNFP	EQU	0CE05H	;*
DINT	EQU	0CE01H
DMOV	EQU	5600H
EINT	EQU	0CE00H
FORT16	EQU	0CE0EH	;*
FORT8	EQU	0CE0FH	;*
IDLE	EQU	0CE1FH	;*
IN0	EQU	8000H
IN1	EQU	8100H
IN2	EQU	8200H
IN3	EQU	8300H
IN4	EQU	8400H
IN5	EQU	8500H
IN6	EQU	8600H
IN7	EQU	8700H
LAC0	EQU	2000H
LAC1	EQU	2100H
LAC2	EQU	2200H
LAC3	EQU	2300H
LAC4	EQU	2400H
LAC5	EQU	2500H
LAC6	EQU	2600H
LAC7	EQU	2700H
LAC8	EQU	2800H
LAC9	EQU	2900H
LAC10	EQU	2A00H
LAC11	EQU	2B00H
LAC12	EQU	2C00H
LAC13	EQU	2D00H
LAC14	EQU	2E00H
LAC15	EQU	2F00H
LACK	EQU	0CA00H
LACT	EQU	4200H	;*
LALK0	EQU	0D001H	;*2
LALK1	EQU	0D101H	;*2
LALK2	EQU	0D201H	;*2
LALK3	EQU	0D301H	;*2
LALK4	EQU	0D401H	;*2
LALK5	EQU	0D501H	;*2
LALK6	EQU	0D601H	;*2
LALK7	EQU	0D701H	;*2
LALK8	EQU	0D801H	;*2
LALK9	EQU	0D901H	;*2
LALK10	EQU	0DA01H	;*2
LALK11	EQU	0DB01H	;*2
LALK12	EQU	0DC01H	;*2
LALK13	EQU	0DD01H	;*2
LALK14	EQU	0DE01H	;*2
LALK15	EQU	0DF01H	;*2
LAR0	EQU	3000H
LAR1	EQU	3100H
LAR2	EQU	3200H
LAR3	EQU	3300H
LAR4	EQU	3400H
LAR5	EQU	3500H
LAR6	EQU	3600H
LAR7	EQU	3700H
LARK0	EQU	0C000H
LARK1	EQU	0C100H
LARK2	EQU	0C200H
LARK3	EQU	0C300H
LARK4	EQU	0C400H
LARK5	EQU	0C500H
LARK6	EQU	0C600H
LARK7	EQU	0C700H
LARP0	EQU	5588H
LARP1	EQU	5589H
LARP2	EQU	558AH
LARP3	EQU	558BH
LARP4	EQU	558CH
LARP5	EQU	558DH
LARP6	EQU	558EH
LARP7	EQU	558FH
LDP	EQU	5200H
LDPK	EQU	0C800H
LDP0	EQU	0C800H
LDP1	EQU	0C801H
LPH	EQU	5300H	;*
LRLK0	EQU	0D000H	;*2
LRLK1	EQU	0D100H	;*2
LRLK2	EQU	0D200H	;*2
LRLK3	EQU	0D300H	;*2
LRLK4	EQU	0D400H	;*2
LRLK5	EQU	0D500H	;*2
LRLK6	EQU	0D600H	;*2
LRLK7	EQU	0D700H	;*2
LST	EQU	5000H
LST1	EQU	5100H	;*
DSPLT	EQU	3C00H
LTA	EQU	3D00H
LTD	EQU	3F00H
LTP	EQU	3E00H	;*
LTS	EQU	5B00H	;*
MAC	EQU	5D00H	;*2
MACD	EQU	5C00H	;*2
MAR	EQU	5500H
MPY	EQU	3800H
MPYA	EQU	3A00H	;*
MPYK	EQU	0A000H
MPYS	EQU	3B00H	;*
MPYU	EQU	0CF00H	;*
DSPNEG	EQU	0CE23H	;*
DSPNOP	EQU	5500H
NORM	EQU	0CEA2H	;*
DSPOR	EQU	4D00H
ORK0	EQU	0D005H	;*2
ORK1	EQU	0D105H	;*2
ORK2	EQU	0D205H	;*2
ORK3	EQU	0D305H	;*2
ORK4	EQU	0D405H	;*2
ORK5	EQU	0D505H	;*2
ORK6	EQU	0D605H	;*2
ORK7	EQU	0D705H	;*2
ORK8	EQU	0D805H	;*2
ORK9	EQU	0D905H	;*2
ORK10	EQU	0DA05H	;*2
ORK11	EQU	0DB05H	;*2
ORK12	EQU	0DC05H	;*2
ORK13	EQU	0DD05H	;*2
ORK14	EQU	0DE05H	;*2
ORK15	EQU	0DF05H	;*2
OUT0	EQU	0E000H
OUT1	EQU	0E100H
OUT2	EQU	0E200H
OUT3	EQU	0E300H
OUT4	EQU	0E400H
OUT5	EQU	0E500H
OUT6	EQU	0E600H
OUT7	EQU	0E700H
OUT8	EQU	0E800H
OUT9	EQU	0E900H
OUT10	EQU	0EA00H
OUT11	EQU	0EB00H
OUT12	EQU	0EC00H
OUT13	EQU	0ED00H
OUT14	EQU	0EE00H
OUT15	EQU	0EF00H
PAC	EQU	0CE14H
DSPPOP	EQU	0CE1DH
POPD	EQU	7A00H	;*
DSPPUSH	EQU	0CE1CH
PSHD	EQU	5400H	;*
DSPRET	EQU	0CE26H
DSPROL	EQU	0CE34H	;*
DSPROR	EQU	0CE35H	;*
RC	EQU	0CE30H	;*
RHM	EQU	0CE38H	;*
RFSM	EQU	0CE36H	;*
RPT	EQU	4B00H	;*
RPTK	EQU	0CB00H	;*
RSXM	EQU	0CE06H	;*
RTC	EQU	0CE32H	;*
RTXM	EQU	0CE20H	;*
RXF	EQU	0CE0CH	;*
ROVM	EQU	0CE02H
SACH0	EQU	6800H
SACH1	EQU	6900H
SACH2	EQU	6A00H
SACH3	EQU	6B00H
SACH4	EQU	6C00H
SACH5	EQU	6D00H
SACH6	EQU	6E00H
SACH7	EQU	6F00H
SACL	EQU	6000H
SACL0	EQU	6000H
SACL1	EQU	6100H
SACL2	EQU	6200H
SACL3	EQU	6300H
SACL4	EQU	6400H
SACL5	EQU	6500H
SACL6	EQU	6600H
SACL7	EQU	6700H
SAR0	EQU	7000H
SAR1	EQU	7100H
SAR2	EQU	7200H
SAR3	EQU	7300H
SAR4	EQU	7400H
SAR5	EQU	7500H
SAR6	EQU	7600H
SAR7	EQU	7700H
SBLK0	EQU	0D003H	;*2
SBLK1	EQU	0D103H	;*2
SBLK2	EQU	0D203H	;*2
SBLK3	EQU	0D303H	;*2
SBLK4	EQU	0D403H	;*2
SBLK5	EQU	0D503H	;*2
SBLK6	EQU	0D603H	;*2
SBLK7	EQU	0D703H	;*2
SBLK8	EQU	0D803H	;*2
SBLK9	EQU	0D903H	;*2
SBLK10	EQU	0DA03H	;*2
SBLK11	EQU	0DB03H	;*2
SBLK12	EQU	0DC03H	;*2
SBLK13	EQU	0DD03H	;*2
SBLK14	EQU	0DE03H	;*2
SBLK15	EQU	0DF03H	;*2
SBRK	EQU	7F00H	;*
SC	EQU	0CE31H	;*
SFL	EQU	0CE18H	;*
SFR	EQU	0CE19H	;*
SFSM	EQU	0CE37H	;*
SHM	EQU	0CE39H	;*
SOVM	EQU	0CE03H
SPAC	EQU	0CE16H
SPH	EQU	7D00H	;*
SPL	EQU	7C00H	;*
SPM0	EQU	0CE08H	;*
SPML1	EQU	0CE09H	;*
SPML4	EQU	0CE0AH	;*
SPMR6	EQU	0CE0BH	;*
SQRA	EQU	3900H	;*
SQRS	EQU	5A00H	;*
SST	EQU	7800H
SST1	EQU	7900H	;*
SSXM	EQU	0CE07H	;*
DSPSTC	EQU	0CE33H	;*
STXM	EQU	0CE21H	;*
SUB0	EQU	1000H
SUB1	EQU	1100H
SUB2	EQU	1200H
SUB3	EQU	1300H
SUB4	EQU	1400H
SUB5	EQU	1500H
SUB6	EQU	1600H
SUB7	EQU	1700H
SUB8	EQU	1800H
SUB9	EQU	1900H
SUB10	EQU	1A00H
SUB11	EQU	1B00H
SUB12	EQU	1C00H
SUB13	EQU	1D00H
SUB14	EQU	1E00H
SUB15	EQU	1F00H
SUBB	EQU	4F00H	;*
SUBC	EQU	4700H
SUBH	EQU	4400H
SUBK	EQU	0CD00H	;*
SUBS	EQU	4500H
SUBT	EQU	04600H	;*
SXF	EQU	0CE0DH	;*
TBLR	EQU	5800H
TBLW	EQU	5900H
TRAP	EQU	0CE1EH	;*
DSPXOR	EQU	4C00H
XORK0	EQU	0D006H	;*2
XORK1	EQU	0D106H	;*2
XORK2	EQU	0D206H	;*2
XORK3	EQU	0D306H	;*2
XORK4	EQU	0D406H	;*2
XORK5	EQU	0D506H	;*2
XORK6	EQU	0D606H	;*2
XORK7	EQU	0D706H	;*2
XORK8	EQU	0D806H	;*2
XORK9	EQU	0D906H	;*2
XORK10	EQU	0DA06H	;*2
XORK11	EQU	0DB06H	;*2
XORK12	EQU	0DC06H	;*2
XORK13	EQU	0DD06H	;*2
XORK14	EQU	0DE06H	;*2
XORK15	EQU	0DF06H	;*2
ZAC	EQU	0CA00H
ZALH	EQU	4000H
ZALR	EQU	7B00H	;*
ZALS	EQU	4100H

;********************************************
; PSP Internal Data Memory Location equates
;********************************************
; Data page 0 (60-7F)
SST0SAVE	EQU	60H	; Timer int handler
SST1SAVE	EQU	61H
HOSTINTR	EQU	62H	; B3h
HOSTACK		EQU	63H	; D3h
SST0SAVE2	EQU	64H	; Host int handler
SST1SAVE2	EQU	65H
SST0SAVE3	EQU	66H	; DAC int handler
SST1SAVE3	EQU	67H
TTEMP		EQU	68H

; External communication area: 1F80-1FFFH (page 63)
HONE		EQU	0	; Constant 1
HTEMP		EQU	1	; Temp var
INT0ACK		EQU	2	; Initialized to int0 ack
ACC0SAVEL	EQU	3	; ACC save during INT0
ACC0SAVEH	EQU	4	;  "	"	"
READY		EQU	5	; DAC ready for update flag
ACC1SAVEL	EQU	6	; ACC save during INT1
ACC1SAVEH	EQU	7	; "	"	"
ACC2SAVEL	EQU	8	; ACC save during INT0
ACC2SAVEH	EQU	9	; "	"	"
PINGPONG	EQU	10	; 0=sample buffer low, 1=sample buffer high
ACC3SAVEL	EQU	11	; ACC save during Timer Int
ACC3SAVEH	EQU	12	;	"	"	"

HOSTFLAG	EQU	127	; 1FFF presence flag AA55H
;*************************************************
DSPCODE EQU THIS WORD
			;Offset
	DW	B	;0	Branch to RESET routine
	DW	INIT
	DW	B	;2	Branch to WFGEN routine
	DW	_INT0
	DW	B	;4	Branch to HOST INT routine
	DW	_INT1
	DW	B	;6	Branch to int 2 routine
	DW	_INT2

	DW 16 DUP (0)	;8-23	reserved

	DW	B	;24	Branch to timer int routine
	DW	_TIMINT
	DW	B	;26	Branch to receive int routine
	DW	_RECINT
	DW	B	;28	Branch to Xmit int routine
	DW	_XMITINT
	DW	B	;30
	DW	_TRAP

_INT2 EQU 32
_RECINT	EQU 32		; Dummy interrupt handler for unused ints
_XMITINT EQU 32
_TRAP	EQU 32
	DW	EINT		;32	Restore ints
	DW	DSPRET		;33	return

;************************************************************************
_INT1 EQU 34			; Host interrupt routine - Ack interrupt
	DW	SST+SST0SAVE2	;0	Save regs
	DW	SST1+SST1SAVE2	;1
	DW	LDPK+63		;2	Page 63
	DW	SACL+ACC1SAVEL	;3
	DW	SACH0+ACC1SAVEH ;4
	DW	LALK0		;5	Write Ack flag 5A5A
	DW	5A5AH		;6
	DW	SACL+HOSTFLAG	;7
	DW	ZALS+ACC1SAVEL	;8	Restore regs
	DW	ADDH+ACC1SAVEH	;9
	DW	LDPK+0		;10
	DW	OUT0+HOSTACK	;11	Output Acknowledgement
	DW	LST1+SST1SAVE2	;12
	DW	LST+SST0SAVE2	;13
	DW	EINT		;14	Ints back on
	DW	DSPRET		;15	and return

;**************************************************************************
_TIMINT EQU _INT1+16		; Timer interrupt routine - interrupt host
	DW	SST+SST0SAVE	;0	Save regs
	DW	SST1+SST1SAVE	;1
	DW	LDPK+63		;2
	DW	SACL+ACC3SAVEL	;3
	DW	SACH0+ACC3SAVEH ;4
				; do other stuff
	DW	LDPK+63		;5
	DW	ZALS+ACC3SAVEL	;6	Restore ACC
	DW	ADDH+ACC3SAVEH	;7
	DW	LDPK+0		;8 	back to page 0
	DW	LST1+SST1SAVE	;9	Restore SSTs
	DW	LST+SST0SAVE	;10
	DW	EINT		;11	Ints back on
	DW	DSPRET		;12	and return

;***********************************************
; DAC Interrupt routine - approx. 344 Hz
;***********************************************
_INT0 EQU _TIMINT+13
	DW	SST+SST0SAVE3	;0	Save SSTs
	DW	SST1+SST1SAVE3	;1
	DW	LDPK+63		;2	Set READY flag
	DW	SACL+ACC0SAVEL	;3	Save Accumulator
	DW	SACH0+ACC0SAVEH ;4
	DW	IN0+HTEMP	;5	Is DAC1 interrupting us?
	DW	LACK+1		;6
	DW	DSPAND+HTEMP	;7
	DW	BZ		;8
	DW	_INT0X		;9
	DW	SACL+READY	;10	Yep, set ready flag (1)
	DW	DSPXOR+PINGPONG	;11	Update pingpong pointer
	DW	SACL+PINGPONG	;12	(0=low, 1=high)
_INT0X EQU _INT0+13
	DW	OUT0+INT0ACK	;13	Ack interrupt
	DW	ZALS+ACC0SAVEL	;14
	DW	ADDH+ACC0SAVEH	;15
	DW	LDPK+0		;16	Now return
	DW	LST1+SST1SAVE3	;17
	DW	LST+SST0SAVE3	;18
	DW	EINT		;19	Ints back on
	DW	DSPRET		;20	and return

;**********************************
; Do whatever work is needed...
;**********************************
DOWORK EQU _INT0+21		;
;
; Set AR's to point to current block to read/write (based on I/O pin)
;
	DW	LRLK0		;0	 Set AR1 = ADC
	DW	4700H		;1
	DW	LRLK6		;2	Set AR6 = DAC0
	DW	4100H		;3
	DW	LRLK7		;4	Set AR7 = DAC1
	DW	4300H		;5
	DW	BIOZ		;6	Bank 1 being updated/read?
	DW	DOWORK2		;7	Yep, set pointers to lower banks
	DW	LRLK0		;8
	DW	4600H		;9
	DW	LRLK6		;10
	DW	4000H		;11
	DW	LRLK7		;12
	DW	4200H		;13
DOWORK2 EQU DOWORK+14

;*********************************
; All done, return to wait loop
;*********************************

	DW	LDPK+63		;0	Page 63
	DW	ZAC		;1	Clear READY flag
	DW	SACL+READY	;2
	DW	B		;3
	DW	FREELOOP	;4

;*************************
;* Initialization Routine
;*************************

INIT	EQU DOWORK2+5
	DW	ROVM		;0	Reset overflow mode
	DW	SSXM		;1	reset sign extension mode
	DW	SPM0		;2	Set P shift mode = 0
	DW	RHM		;3	reset Hold Mode
	DW	CNFD		;4	Config mem = Data

	DW	LRLK0		;5	Clear sample buffers
	DW	4000H		;6
	DW	LARP0		;7	select AR0
	DW	ZAC		;8
	DW	RPTK+255	;9
	DW	SACL+ARPLUS	;10	DAC2 0
	DW	RPTK+255	;11
	DW	SACL+ARPLUS	;12	DAC2 1
	DW	RPTK+255	;13
	DW	SACL+ARPLUS	;14	DAC1 0
	DW	RPTK+255	;15
	DW	SACL+ARPLUS	;16	DAC1 1

INIT2 EQU INIT+17

	DW	LDPK+0		;0	Set HOSTINTR
	DW	LACK+0B3H	;1
	DW	SACL+HOSTINTR	;2
	DW	LACK+0D3H	;3	Set HOSTACK
	DW	SACL+HOSTACK	;4
	DW	LALK0		;5	Timer period
	DW	16666		;6
	DW	SACL+3		;7	(TIMEPERIOD)
	DW	SACL+2		;8	(TIMERREG)
	DW	LDPK+63		;9
	DW	LACK+0E3H	;10
	DW	SACL+INT0ACK	;11
	DW	LALK0		;12	Write present flag
	DW	55AAH		;13
	DW	SACL+HOSTFLAG	;14
;
; Enable DACs & Ints
INIT3 EQU INIT2+15
	DW	LDPK+0		;0	Set IMR = TINT, INT1, INT0
	DW	LACK+0BH	;1	 (INT1=Host, INT0=SP/B)
	DW	SACL+4		;2
	DW	LDPK+63		;3
	DW	LACK+73H	;4	Reset Cntr, enable DAC's
	DW	SACL+HTEMP	;5
	DW	OUT0+HTEMP	;6
	DW	LACK+1		;7
	DW	SACL+READY	;8	Set ready flag
	DW	EINT		;9

FREELOOP EQU INIT3+10		; Page=63, AR=?
	DW	LAC0+READY	;0	Is DAC ready
	DW	BNZ		;1	If so, go do it	2
	DW	DOWORK		;2
	DW	B		;3	Endless loop
	DW	FREELOOP	;4

DSPCODE_END EQU THIS BYTE
;*************************************

_DATA	ENDS

_IOSEG	SEGMENT PUBLIC 'IOSEG_CODE'

ACPAREAD PROC FAR	; unsigned int far pascal acparead(unsigned address);
	PUBLIC	ACPAREAD
	PUSH	BP
	MOV	BP,SP
	MOV	DX,_ACPABASE
	ADD	DX,ADDREGLOW			; DX = address reg low
	MOV	AX,+6[BP]
	OUT	DX,AX
	SUB	DX,ADDREGLOW-DATAREGLOW		; DX = data reg low
	IN	AX,DX
	POP	BP
	RET	2
ACPAREAD ENDP

ACPAWRITE PROC FAR	; unsigned int far pascal acpawrite(unsigned address, unsigned data);
	PUBLIC	ACPAWRITE
	PUSH	BP
	MOV	BP,SP
	MOV	DX,_ACPABASE		; Set DX = address reg low
	ADD	DX,ADDREGLOW
	MOV	AX,+8[BP]
	OUT	DX,AX
	SUB	DX,ADDREGLOW-DATAREGLOW	; DX = data reg low
	MOV	AX,+6[BP]
	OUT	DX,AX
	POP	BP
	RET	4
ACPAWRITE ENDP

ACPACLOSE PROC FAR	; far pascal acpaclose();
	PUBLIC	ACPACLOSE
	MOV	DX,_ACPABASE
	ADD	DX,HOSTCOMREG
	SUB	AL,AL
	OUT	DX,AL			; Reset the ACPA
	RET
ACPACLOSE ENDP

;
; This routine will fill the ACPA memory with a constant, and verify that
; every word can be read correctly.
;
DSPMTEST PROC NEAR			; AX = Value read/verify
	MOV	BX,AX
	SUB	AX,AX			; Set ACPA address = Code (0)
	SUB	AX,AX
	MOV	DX,_ACPABASE
	ADD	DX,ADDREGLOW
	OUT	DX,AX
	JMP	$+2
	MOV	CX,8192
	SUB	DX,ADDREGLOW-DATAREGLOW
	MOV	AX,BX
MTEST1:					; Write pattern to memory
	OUT	DX,AX
	LOOP	MTEST1
	JMP	$+2
					; Now verify that pattern is there
	SUB	AX,AX			; Set ACPA address = Code (0)
	ADD	DX,ADDREGLOW
	OUT	DX,AX
	JMP	$+2
	MOV	CX,8192
	SUB	DX,ADDREGLOW-DATAREGLOW
MTEST2:
	IN	AX,DX
	CMP	AX,BX
	JNE	MTESTERR
	LOOP	MTEST2
MTESTERR:
	MOV	AX,CX			; Return counter (0 if ok)
	RET
DSPMTEST ENDP

;*************************************************************************
;			ACPAINIT
;
; This IOPL routine will verify the presence of the ACPA at the ACPABASE
; address set in sample1.c or sample2.c.
;
ACPAINIT PROC	FAR	; far pascal acpainit();
	PUBLIC	ACPAINIT
	PUSH	SI
	CLD
;
; Enable ACPA
;
	MOV	DX,_ACPABASE
	ADD	DX,HOSTCOMREG
	MOV	AL,08H			; Reset DSP, disable ints
	OUT	DX,AL
	JMP	$+2
	INC	DX			; Clear ACC ERR
	OUT	DX,AL
;
; Write all ones out to entire 8k memory, and verify each word
;
	MOV	AX,0FFFFH
	CALL	DSPMTEST
	OR	AX,AX
	JZ	TEST1_OK
	POP	SI
	MOV	AX,1			; RC=1 FFFF memory test failed
	RET

TEST1_OK:
;
; Write AA55 test pattern to entire 8k memory, and verify each word
;
	MOV	AX,0AA55H
	CALL	DSPMTEST
	OR	AX,AX
	JZ	TEST2_OK
	POP	SI
	MOV	AX,2			; RC=2 AA55 memory test failed
	RET
TEST2_OK:
;
; Zero out and verify each word
;
	MOV	AX,0
	CALL	DSPMTEST
	OR	AX,AX
	JZ	TEST3_OK
	POP	SI
	MOV	AX,3			; Ret RC=3 0000 memory test failed
	RET
TEST3_OK:

;*********************************
; Load code at 0000 (8k words max)
;*********************************
	SUB	AX,AX			; Set ACPA address = Code (0)
	MOV	DX,_ACPABASE
	ADD	DX,ADDREGLOW
	OUT	DX,AX
	JMP	$+2
;
; Load the DSP code contained between labels DSPCODE and DSPCODE_END
;
	MOV	SI,OFFSET DGROUP:DSPCODE	; Set DS:SI = DSP code
	MOV	CX,OFFSET DGROUP:DSPCODE_END	; Set CX = length of DSP code
	SUB	CX,SI
	SHR	CX,1				; divide by 2 for words
	SUB	DX,ADDREGLOW-DATAREGLOW		; and here it is
ACPA0:
	LODSW					; Copy data
	OUT	DX,AX				;  to ACPA data reg
	LOOP	ACPA0				;  and loop for entire length
;
; Enable ACPA
;
	ADD	DX,HOSTCOMREG
	MOV	AL,09H			; Remove Reset bit 0, leave int disabled
	OUT	DX,AL
	JMP	$+2
	INC	DX
	OUT	DX,AL			; Clear ACC ERR (if any)
;
; Wait for DSP to respond by writing to address 1FFFH
;
	MOV	CX,-1			; Wait loop count
DSPWAIT:
	MOV	DX,_ACPABASE
	ADD	DX,ADDREGLOW
	MOV	AX,1FFFH
	OUT	DX,AX
	JMP	$+2
	SUB	DX,ADDREGLOW-DATAREGLOW
	IN	AX,DX
	CMP	AX,55AAH		; Is
	JE	HAVEFLAG
	LOOP	DSPWAIT
	MOV	AX,4			; Return error RC=4 (55AA not set)
	POP	SI
	RET

HAVEFLAG:

; Verify that DSP responds to a Host Interrupt by changing flag to 5A5A
	ADD	DX,HOSTCOMREG
	MOV	AL,03H
	OUT	DX,AL
	JMP	$+2
	MOV	CX,-1
TESTLOOP1:
	MOV	AX,1FFFH
	MOV	DX,_ACPABASE
	ADD	DX,ADDREGLOW
	OUT	DX,AX
	JMP	$+2
	SUB	DX,ADDREGLOW-DATAREGLOW
	IN	AX,DX
	CMP	AX,5A5AH		; Is it 5A5A?
	JE	HAVEACK
	LOOP	TESTLOOP1
	MOV	AX,5			; Return error RC=5 (No DSP int ack)
	POP	SI
	RET
HAVEACK:
	SUB	AX,AX		; Return 0 (everything looks ok)
	POP	SI
	RET
ACPAINIT ENDP

_IOSEG	ENDS
	END
